#include "NFC.h"
#include <SPI.h>
#include <PubSubClient.h>
#include <DESFire.h>
#include "helpers.h"

NFC::NFC(int pin_ss, int pin_rst)
{
    rfid = new DESFire(pin_ss, pin_rst);

    SPI.begin();
    rfid->PCD_Init();
    rfid->PCD_DumpVersionToSerial();
}

bool NFC::requestCard()
{
    byte bufferATQA[2];
	byte bufferSize = sizeof(bufferATQA);
	rfid->PCD_WriteRegister(rfid->TxModeReg, 0x00);
	rfid->PCD_WriteRegister(rfid->RxModeReg, 0x00);
	rfid->PCD_WriteRegister(rfid->ModWidthReg, 0x26);

	MFRC522::StatusCode status = rfid->PICC_RequestA(bufferATQA, &bufferSize);
	Serial.printf("PICC_RequestA: 0x%02x\n", status);
	return status == MFRC522::STATUS_OK;
}

MFRC522::Uid NFC::getUID()
{
    return uid;
}

bool NFC::selectCard()
{
    MFRC522::StatusCode status = rfid->PICC_Select(&uid);
    Serial.printf("PICC_Select: 0x%02x\n", status);
    if(status != MFRC522::STATUS_OK)
    {
        return false;
    }

    if(rfid->uid.sak != 0x20)
    {
        Serial.printf("SAK Incorret: 0x%02x\n", rfid->uid.sak);
    }

    


    
    byte ats_buffer[16];
    byte ats_buffer_len = 16;




    status = rfid->PICC_RequestATS(ats_buffer, &ats_buffer_len);
    if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PICC_RequestATS: 0x%02x\n", status);
        return false;
    }

    status = rfid->PICC_ProtocolAndParameterSelection(0x00, 0x11);
    if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PICC_ProtocolAndParameterSelection: 0x%02x\n", status);
        return false;
    }

    cardSelected = true;
    uid = rfid->uid;
    Serial.println("Card Selected");
    return true;
}

bool NFC::deselectCard()
{
    cardSelected = false;
    Serial.println("Card Deselected");

    return true;
}

bool NFC::hasCardSelected()
{
    return cardSelected;
}

bool NFC::isCardLost()
{
    uint8_t control = 0x00;
    for(int i=0; i<3; i++)
    {
        if( !rfid->PICC_IsNewCardPresent() )
        {
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            if(rfid->PICC_ReadCardSerial())
            {
                control |= 0x16;
            }
            control += 0x1;
        }
        control += 0x4;
    }

    Serial.printf("Control: %d\n", control);

    if(!(control == 13 || control == 14))
    {
        Serial.println("Card Lost");
        return true;
    }
    else
    {
        return false;
    }
}

MFRC522::StatusCode NFC::Transceive(byte* command, byte command_len, byte* response, byte* response_len)
{
    byte buffer[63 + 4];
    byte buffer_len = command_len + 4;
    MFRC522::StatusCode status;

    buffer[0] = 0x0A;
	buffer[1] = 0x00;
    memcpy(&buffer[2], command, command_len);

    status = rfid->PCD_CalculateCRC(buffer, buffer_len - 2, &buffer[buffer_len - 2]);
	if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PCD_CalculateCRC: 0x%02x\n", status);
		return status;
	}

    Serial.println(buffer_len);
    printbytes(buffer, buffer_len);
    
    status = rfid->PCD_TransceiveData(buffer, buffer_len, response, response_len, NULL, 0, true);
	if (status != MFRC522::STATUS_OK)
    {
        Serial.printf("PCD_TransceiveData: 0x%02x\n", status);
		return status;
	}

    return status;
}