#include <SPI.h>
#include <MFRC522.h>
#include <Desfire.h>

#include "Pins.h"

DESFire mfrc522(PIN_RFID_SPI_SS, PIN_RFID_RST);  // Create MFRC522 instance

void setup() 
{
    Serial.begin(115200);   // Initialize serial communications with the PC
    while (!Serial);    // Do nothing if no serial port is opened (added for Arduinos based on ATMEGA32U4)
    SPI.begin();      // Init SPI bus
    mfrc522.PCD_Init();   // Init MFRC522
    mfrc522.PCD_DumpVersionToSerial();  // Show details of PCD - MFRC522 Card Reader details
    Serial.println(F("Scan PICC to see UID, SAK, type, and data blocks..."));
}

void printBuffer(byte* buffer, byte buffer_len)
{
    Serial.printf("Len: %02d Buffer: 0x", buffer_len);
    for(int i = 0; i < buffer_len; i++)
    {
        Serial.printf("%02x", buffer[i]);
    }
    Serial.println();
}

void loop() 
{
    // RequestA
    if ( ! mfrc522.PICC_IsNewCardPresent()) 
    {
        return;
    }

    // PICC_Select ???
    if ( ! mfrc522.PICC_ReadCardSerial())
    {
        return;
    }

    // Check for DESFire
    if (mfrc522.uid.sak != 0x20) 
    {
        // Dump debug info about the card; PICC_HaltA() is automatically called
        mfrc522.PICC_DumpToSerial(&(mfrc522.uid));
        return;
    }

    Serial.println("New Card");
    MFRC522::StatusCode state;
    
    // RATS
    byte ats[16];
    byte atsLength = 16;
    state = mfrc522.PICC_RequestATS(ats, &atsLength);
    if (state != MFRC522::STATUS_OK) {
        Serial.println(F("Failed to get ATS!"));
        Serial.println(state);
        mfrc522.PICC_HaltA();
        return;
    }

    // PPS
    state = mfrc522.PICC_ProtocolAndParameterSelection(0x00, 0x11, 0x00);
    if (state != MFRC522::STATUS_OK) 
    {
        Serial.println(F("Failed to perform protocol and parameter selection (PPS)!"));
        Serial.println(state);
        mfrc522.PICC_HaltA();
        return;
    }

    Serial.println("Complete ISO 14443 Activation");

    DESFire::mifare_desfire_tag tag;
    tag.pcb = 0x0A;
    tag.cid = 0x00;

    DESFire::StatusCode des_state;
    byte buffer[64] = {0x5A, 0x00, 0x00, 0x03, 0x42, 0x41, 0x46, 0x00};
	byte bufferSize = 8;

    byte response_buffer[64];
    byte response_len;

    des_state = mfrc522.MIFARE_BlockExchangeWithData(&tag, 0x90, buffer, &bufferSize, response_buffer, &response_len);
    state = des_state.mfrc522;
	if (state != MFRC522::STATUS_OK) 
    {
		Serial.println("Data Exchange failed");
        Serial.println(state);
	}
    Serial.println("Data Exchange complete");
    printBuffer(response_buffer, response_len);

    // Deselect
    // Missing

    // HaltA
    mfrc522.PICC_HaltA();
    Serial.println("Stop Connection");
}